macro_rules! extract_arg {
    ($ctx:ident, $name:literal, String) => {
        $ctx.args.iter().find(|opt| opt.name == $name).map(|opt| &opt.value).map_or_else(
            || String::new(),
            |v| match v {
                poise::serenity_prelude::ResolvedValue::String(s) => s.to_string(),
                _ => String::new(),
            },
        )
    };
    ($ctx:ident, $name:literal, Option<String>) => {
        $ctx.args
            .iter()
            .find(|opt| opt.name == $name)
            .map(|opt| &opt.value)
            .map(|v| match v {
                poise::serenity_prelude::ResolvedValue::String(s) => Some(s.to_string()),
                _ => None,
            })
            .flatten()
    };
    ($ctx:ident, $name:literal, bool) => {
        $ctx.args.iter().find(|opt| opt.name == $name).map(|opt| &opt.value).map_or(false, |v| {
            match v {
                poise::serenity_prelude::ResolvedValue::Boolean(b) => b.to_owned(),
                _ => false,
            }
        })
    };
    ($ctx:ident, $name:literal, Option<bool>) => {
        $ctx.args
            .iter()
            .find(|opt| opt.name == $name)
            .map(|opt| &opt.value)
            .map(|v| match v {
                poise::serenity_prelude::ResolvedValue::Boolean(b) => Some(b.to_owned()),
                _ => None,
            })
            .flatten()
    };
}

use proc_macro::TokenStream;
use syn::parse::Parser;

#[proc_macro_derive(Extract)]
pub fn extract(input: TokenStream) -> TokenStream {
    // Construct a string representation of the type definition
    let s = input.to_string();

    // Parse the string representation
    let ast = syn::parse_derive_input(&s).unwrap();

    // Build the impl
    let gen = impl_extract(&ast);

    // Return the generated impl
    gen.parse().unwrap()
}

fn impl_extract(ast: &syn::DeriveInput) -> TokenStream {
    let name = &ast.ident;
    TokenStream::from(quote::quote! {
        impl Extract for #name {
            fn extract(ctx: ) -> Self {
                println!("Hello, World! My name is {}", stringify!(#name));
            }
        }
    })
}
